<?php

namespace App\Http\Controllers\Admin;

use App\Filter\FilterClass\Company\Email;
use App\Filter\FilterClass\Company\Industry;
use App\Filter\FilterClass\Company\PhoneNumber;
use App\Filter\FilterClass\Company\Q;
use App\Filter\FilterClass\Company\ReferralCode;
use App\Filter\FilterClass\Company\Status;
use App\Filter\FilterClass\Users\LastName;
use App\Filter\FilterClass\Users\WorkExperience;
use App\Http\Controllers\Controller;
use App\Http\Resources\AdminDashboard;
use App\Models\Company;
use App\Models\Job;
use App\Models\Project;
use App\Models\User;
use App\Traits\ApiResponser;
use App\Traits\Helper;
use App\Traits\pipeline;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;

class DashboardController extends Controller
{
    use Helper, ApiResponser, pipeline;

    public function __construct()
    {
        $this->middleware(['auth:sanctum', 'isAdmin']);
    }

    public function index()
    {
        $user = User::all();
        $company = Company::all();
        $userAndCompany = Arr::collapse([$user, $company]);
//        return $userAndCompany;
        $users = null;
//        if ($user){
//            foreach ($user as $item){
//                $users[]=[
//                    'id'=>$item->id,
//                    'type'=>1,
//                    'job_title_id'=>$item->job_title_id,
//                    'email'=>$item->email,
//                    'name'=>$item->name,
//                    'last_name'=>$item->last_name,
//                    'phone_number'=>$item->phone_number,
//                    'image'=>$item->image,
//                    'about_us'=>$item->about_us,
//                    'work_experience'=>$item->work_experience,
//                    'familiar_method'=>$item->familiar_method,
//                    'referral_code'=>$item->referral_code,
//                    'status'=>$item->status,
//                    'email_verified_at'=>$item->email_verified_at,
//                    'created_at'=> $item->created_at,
//                ];
//            }}

//    $Company=Company::all();
//        return $Company;
//    $r=Comment::all();
//
//        return ($userAndCompany)?$this->successResponseWithoutMessage(true, AdminDashboard::collection($this->paginate($userAndCompany,20)))
//        :$this->failedResponse('No information found');
//        return DB::table('users')
//            ->select(DB::raw('Month(created_at) as `data`'))
////            ->groupby('monthyear')
//            ->get();


        $countTechnician = array();
        $countUser = array();

        for ($i = 0; $i <= 11; $i++) {
            $v = now()->addMonth($i);
            $countTechnician[] = ['time' => $v->format('F'), 'count' => User::whereDate('created_at', '>=', $v->startOfMonth())->whereDate('created_at', '<=', $v->endOfMonth())->count()];
            $countUser[] = ['time' => $v->format('F'), 'count' => Company::whereDate('created_at', '>=', $v->startOfMonth()->format('Y-m-d'))->whereDate('created_at', '<=', $v->endOfMonth()->format('Y-m-d'))->count()];
        }
        $dashboard['analyze'] = ['company' => $countUser, 'user' => $countTechnician];
        $dashboard['register'] = [
            'user' => User::count(),
                'company' => Company::count(),
                'project'=>Project::count(),
                'job'=>Job::count(),
            ] ?? null;
        $dashboard['last']=[
            'users'=>User::query()->latest('created_at')->take(10)->get(),
            'companies'=>Company::query()->latest('created_at')->take(10)->get(),
            'projects'=>Project::query()->latest('created_at')->take(10)->get(),
        ];
        return ($dashboard) ? $this->successResponseWithoutMessage(true, AdminDashboard::collection($dashboard))
            : $this->failedResponse('No information found');
    }

    public function userList()
    {
//        $user = User::all();
        $user = $this->pipeline(User::query()->with('JobTitle'), [
            \App\Filter\FilterClass\Users\Q::class,LastName::class, PhoneNumber::class, Industry::class,
            Status::class, ReferralCode::class, Email::class,
        ]);
        return ($user) ? $this->successResponseWithoutMessage(true, $user)
            : $this->failedResponse('No information found');
    }

    public function companyList()
    {
        $company = $this->pipeline(Company::query()->with('JobTitle'), [
            Q::class, PhoneNumber::class, Industry::class,
            Status::class, ReferralCode::class, Email::class,WorkExperience::class
        ]);
        return ($company) ? $this->successResponseWithoutMessage(true, $company)
            : $this->failedResponse('No information found');
    }

}
