<?php

namespace App\Http\Controllers\Admin\Structure;

use App\Filter\DateFrom;
use App\Filter\DateTo;
use App\Filter\FilterClass\Comments\Email;
use App\Filter\FilterClass\Comments\FirstName;
use App\Filter\FilterClass\Comments\Status;
use App\Filter\FilterClass\Users\LastName;
use App\Http\Controllers\Controller;
use App\Http\Requests\UpdateAdminCommentsRequest;
use App\Models\Comment;
use App\Traits\ApiResponser;
use App\Traits\pipeline;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

class CommentsController extends Controller
{
    use pipeline,ApiResponser;
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function __construct()
    {

        $this->middleware(['auth:sanctum', 'isAdmin']);
//
        $this->middleware('create-comments')->only(['store']);
        $this->middleware('can:edit-comments')->only(['update']);
        $this->middleware('can:read-comments')->only(['index','show']);
        $this->middleware('can:delete-comments')->only(['delete']);
    }
    public function index()
    {
        $Comments= $this->pipeline(
            Comment::query()->with('profile','client'),[FirstName::class,LastName::class,
                Email::class,Status::class,DateFrom::class,DateTo::class,
        ]);
        return (isset($Comments))?$this->onSuccess($Comments):$this->onFaile('comment not found');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $Comments=
            Comment::query()->where('id',$id)->with('profile','client')->get();
        return (isset($Comments))?$this->onSuccess($Comments):$this->onFaile('comment not found');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(UpdateAdminCommentsRequest $request, $id)
    {
      $comments=  Comment::find($id);
//      return $request;
        $requests=$request->all();
        $requests['client_type']='App\\Models\\' . Str::ucfirst($request->client_type);
        $requests['profile_type']='App\\Models\\' . Str::ucfirst($request->profile_type);
//        return $requests['client_type']::find($requests['client_id']);
//        return $requests['parent_id'];
            if (isset($requests['parent_id']))
        if (!(Comment::query()->where('id',$requests['parent_id'])->first()))
            return $this->onFaile('parent comment not found');
        if (($requests['profile_type']==$requests['client_type'])&&$requests['profile_id']==$requests['client_id'])
            return $this->onFaile('The commenter is the recipient');
        if(!($requests['client_type']::find($requests['client_id'])))
            return $this->onFaile('client not found');
        if(!($requests['profile_type']::find($requests['profile_id'])))
            return $this->onFaile('profile not found');
//        return  $requests;
      if (!$comments)
          return $this->onFaile('comments not found ');

          $comments->update($requests);
          return $this->onSuccess(Comment::query()->with('client','profile')->find($id));

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $Comments = Comment::find($id);
        return ($Comments)
            ?  $this->onSuccess($Comments->delete()):
            $this->onFaile('comments not found');
    }
//    public function
}
