<?php

namespace App\Http\Controllers\Admin\Structure;

use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\FaqsRequest;
use App\Repositories\Admin\Structure\FAQRepository;
use App\Traits\ApiResponser;
use Illuminate\Http\Request;

class FAQController extends Controller
{
    use ApiResponser;

    public $request;
    public $repository;

    public function __construct(Request $request, FAQRepository $repository)
    {
        $this->repository = $repository;
        $this->request = $request;
        $this->middleware(['auth:sanctum','isAdmin']);

        $this->middleware('can:create-faq')->only(['store']);
        $this->middleware('can:edit-faq')->only(['update']);
        $this->middleware('can:read-faq')->only(['index']);
        $this->middleware('can:delete-faq')->only(['delete']);
    }

    public function store(FaqsRequest $FAQRequest)
    {
        $repo = $this->repository->createFAQ($FAQRequest->question, $FAQRequest->answer);
        return ($repo)?$this->successResponse(true, $repo, 'add FAQ successfully'):
            $this->failedResponse($repo);
    }

    public function update(FaqsRequest $FAQRequest,$id)
    {
        $repo = $this->repository->updateFAQ($id, $FAQRequest->question, $FAQRequest->answer);
        return ($repo)?$this->successResponse(true, $repo, 'update FAQ successfully'):
            $this->failedResponse('failed update FAQ');
    }

    public function destroy($id)
    {
        $repo = $this->repository->deleteFAQ($id);
        return ($repo)?$this->successResponse(true, null, 'delete FAQ successfully'):
            $this->failedResponse('failed delete FAQ ');
    }

    public function index()
    {
        $repo = $this->repository->showFAQ();
        return $this->successResponse(true, $repo, 'send FAQ successfully');
    }

}
