<?php

namespace App\Http\Controllers\Admin\Structure;

use App\Filter\FilterClass\project\Name;
use App\Filter\FilterClass\project\ReferenceNumber;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ImageController;
use App\Http\Controllers\OwnerController;
use App\Http\Controllers\SpecialCommentController;
use App\Http\Resources\Admin\ProjectsResource;
use App\Http\Resources\ListProjectOnlyShowResource;
use App\Http\Resources\PaginationResource;
use App\Http\Resources\SetProjectResource;
use App\Models\Address;
use App\Models\GeneralContractor;
use App\Models\Image;
use App\Models\Owner;
use App\Models\ParticipantsProject;
use App\Models\Project;
use App\Models\SpecialComment;
use App\Models\StyleImage;
use App\Services\AddressService;
use App\Services\GeneralContractorService;
use App\Traits\ApiResponser;
use App\Traits\Helper;
use App\Traits\pipeline;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class ProjectController extends Controller
{
    use pipeline,ApiResponser,Helper;
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
//        return 1;
        $project= $this->pipeline(Project::query()
            ->with('images','specialComments','projectable','address','participants','owner','generalContractor')
            ,[Name::class,ReferenceNumber::class]);
//        return  $project;
        return ($project)?$this->onSuccess(new  PaginationResource($project,ProjectsResource::collection($project))):$this->onSuccess('error');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $project=   Project::query()->where('id',$id)->with('images','specialComments','address','participants','owner','generalContractor')->get();
        return ($project)?$this->onSuccess($project):$this->onFaile('project not found');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $projectRequest, $id)
    {

//        if ($projectRequest->image) {
//
//            if (is_array($projectRequest->image)) {
//
//                foreach ($projectRequest['image'] as $keys => $images) {
////                    return $projectRequest['image'] ;
//
//                    $image = Storage::exists('/temp/images/' . $images['name']);
//                    if (!$image)
//                        return $this->onFaile('photo not found ');
//                }
//            } else {
//                return $this->onFaile('invalid photo ');
//            }
//        }

        if ($projectRequest['project_general_contractor.image']) {
            $image = Storage::exists('/temp/images/' . $projectRequest['project_general_contractor.image']);
            if (!$image)
                return $this->onFaile(' general contractor photo not find');

        }

//        return $projectRequest;
        //add multi member
        $existsClients=ParticipantsProject::query()->where('participants_project_id',$projectRequest['member_id'])->where('participants_project_type',$projectRequest['member_type'])  ->where('project_id', $id)->get();

        $project = Project::where('id', $id)->first();
        if (!$project)
            return $this->onFaile('project not found');
//        $this->setInformation($clientRole);
        if ($projectRequest['project_general_contractor.id_general']&&$projectRequest['project_general_contractor.type']) {
            $client=('App\\Models\\' . Str::ucfirst($projectRequest['project_general_contractor.type']))::find($projectRequest['project_general_contractor.id_general']);
            if (!$client) {
                return $this->onFaile('general contractor not found ');
            }
                if ($client->job_title_id != 1)
                    return $this->onFaile('The job title of the contracting industry is not public');
        }

//return 1;
//        if ($this->request['image']) {
//            $images = Image::where('imageable_type', Project::class)->where('imageable_id', $id)->get();
//          return $images;
//            if (($images)->isNotEmpty()) {
////                return 1;
//                foreach ($images as $image) {
////                    return $image;
//                    if (Storage::exists('public/images/' . $image->name))
//                        Storage::delete('public/images/' . $image->name);
//                $image->delete();
//                    $keyword =ImageKeyword::where('image_id',$image->id)->delete();
//                }
//
//            }
//            return $projectRequest['project_address'];
        if ($projectRequest['project_address']) {
            Address::where('addressable_id', $id)->where('addressable_type', Project::class)->delete();
            $addressService = new AddressService();
            $address = $addressService->store($projectRequest['project_address'], Project::class, $id);
        }
        $project->update([
            'name' => $projectRequest['project_name'] ?? null,
            'type' => $projectRequest['project_type'] ?? null,
            'condition' => $projectRequest['project_condition'] ?? "",
            'priority' => $projectRequest['project_priority'] ?? null,
            'style' => $projectRequest['project_style_id'] ?? null,
            'start_date' => $projectRequest['project_start'] ?? "000-00-00",
            'end_date' => $projectRequest['project_end'] ?? null,
        ]);
//                        return  is_array($projectRequest->image);
//        return $projectRequest ;
//        if ($projectRequest->image && is_array($projectRequest->image)) {
//
//            $this->addPhotoToProject($id,$projectRequest->image);
//        }
//        return 2;

        if ($projectRequest->project_general_contractor) {
            $clientProject = ParticipantsProject::query()
                ->where('project_id', $id)
                ->where('role', ParticipantsProject::GENERAL_CONTRACTOR_ROLE)->first();
//           return $clientProject;
//            ParticipantsProject::query()
//                ->where('project_id', $id)
//                ->where('role', ParticipantsProject::GENERAL_CONTRACTOR_ROLE)
//                ->delete();
            $GeneralContractorList = GeneralContractor::query()
                ->where('project_id', $id)->pluck('id');
            GeneralContractor::query()
                ->where('project_id', $id)->delete();
//            return 10;
            $specialComment = new SpecialCommentController();
//return $clientProject;
            if ($projectRequest['project_general_contractor.id_general'] && $projectRequest['project_general_contractor.type']) {
//                return 526;
                if ($clientProject)
                    SpecialComment::where('project_id', $id)->where('profile_type', $clientProject->participants_project_type)->where('profile_id', $clientProject->participants_project_id)->delete();

                $table = DB::table('participants_project')->insert([
                    'participants_project_type' => $this->getPathClass($projectRequest['project_general_contractor.type']),
                    'participants_project_id' => $projectRequest['project_general_contractor.id_general'],
                    'project_id' => $project->id,
                    'role' => ParticipantsProject::GENERAL_CONTRACTOR_ROLE,

//            'job_title_id' => $this->request['job_title_id'],
                    'start_date' => $projectRequest['start_date'],
                    'end_date' => $projectRequest['start_date'],
                ]);
//            return 3;
//          return  $this->request['project_general_contractor.id_general'];
            } else {

//              return $owners;
//return $GeneralContractorList;
//                GeneralContractor::query()
//                    ->where('project_id', $id)->delete();
//                return 4;
                $general = new  GeneralContractorService();
                $generalContactor = $general->storeUpdate($projectRequest['project_general_contractor'], $id);
                $generalContactor['type'] = GeneralContractor::class;


//                if ($projectRequest['project_general_contractor.text'] && $projectRequest['project_general_contractor.rate']) {
////                    SpecialComment::where('project_id',$id)->whereIn('profile_id',$owners)->where('profile_type',Owner::class)->delete();
//                    SpecialComment::where('project_id', $id)->whereIn('profile_id', $GeneralContractorList)->where('profile_type', GeneralContractor::class)->delete();
//
////                return 1;
//                    if ($projectRequest['project_general_contractor.id_general'] && $projectRequest['project_general_contractor.type']) {
////                return 2;
//                        $specialComment->store($projectRequest['project_general_contractor'], $id, $this->clientType, $this->clientId,$projectRequest['project_general_contractor.id_general'], $this->getPathClass($projectRequest['project_general_contractor.type']));
//                    } else {
////                return $generalContractor;
//                        if ($generalContactor)
//                            $specialComment->store($projectRequest['project_general_contractor'], $id, $this->clientType, $this->clientId, $generalContactor->id, $generalContactor->type);
//                    }
//                }
            }
        }
//                return 2;

        if ($projectRequest->project_owner) {
            ParticipantsProject::query()
                ->where('project_id', $id)
                ->where('role', ParticipantsProject::OWNER_ROLE)
                ->delete();
            if ($projectRequest['project_owner.id'] && $projectRequest['project_owner.type']) {
//                return 50;
                $table = DB::table('participants_project')->insert([
                    'participants_project_type' => $this->getPathClass($projectRequest['project_owner.type']),
                    'participants_project_id' => $projectRequest['project_owner.id'],
                    'project_id' => $project->id,
//            'job_title_id' => $this->request['job_title_id'],
                    'role' => ParticipantsProject::OWNER_ROLE,
                    'start_date' =>$projectRequest['start_date'],
                    'end_date' => $projectRequest['end_date'],
                ]);

            } else {
                $owners = Owner::query()
                    ->where('ownerable_id', $id)->where('ownerable_type', Project::class)->pluck('id');
//              return $owners;
                SpecialComment::where('project_id', $id)->whereIn('profile_id', $owners)->where('profile_type', Owner::class)->delete();

                Owner::query()
                    ->where('ownerable_id', $id)->where('ownerable_type', Project::class)->delete();
//                return $owners;


                $ownerController = new OwnerController();
                $owner = $ownerController->store($projectRequest['project_owner'], 'App\Models\Project', $id);
                $owner['type'] = Owner::class;


//                if ($this->request['project_owner.text'] && $this->request['project_owner.rate']) {
////                return 1;
////                    if ($this->request['project_owner.id'] && $this->request['project_owner.type']) {
////                return 2;
////                        $specialComment->store($this->request['project_owner'], $id, $this->clientType, $this->clientId, $this->request['project_owner.id'], $this->getPathClass($this->request['project_owner.type']));
////                    } else {
////                return $generalContractor;
////                        SpecialComment::where('client_id',$this->clientId)->where('client_type',$this->clientType)->where('profile_type',$this->clientType)->
//                    $specialComment->store($this->request['project_owner'], $id, $this->clientType, $this->clientId, $owner->id, $owner->type);
////                    }
//                }

            }
        }


//return 5555;

        return $this->successResponse(true, SetProjectResource::make(Project::where('id', $id)->with(['address', 'images', 'owner', 'generalContractor',
            'specialComments'
            ])->first()), 'add successfully');

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $project = Project::find($id);
//        return $project;
        if (!$project)
    return    $this->onFaile('project not found');
        $project->specialComments()->delete();
        $project->participants()->delete();
        $project->address()->delete();
        $project->images()->delete();
        $project->generalContractor()->delete();
        $project->owner()->delete();
        $project->delete();
        return ($project)
            ?  $this->onSuccess(true):
            $this->onFaile('project not found');
    }
    public function addPhotoToProject($id,$image)
    {
        if ($image) {
            $imageController = new ImageController();

            foreach ($image as $keys => $images) {
                //dd($this->request['image']);
                //dd($images);
//                return $images['style'];

                $imageName = $imageController->move($images['name']);
                if (!isset($images['link'])) {
                    $images['link'] = null;
                }
                $insertImage = $imageController->insertImage($images['name'], Project::class, $id, null, null, $images['link']);
                $imageIds[$keys] = $insertImage->toArray()['id'];
//                return $imageIds[$keys] ;
                if (isset($images['tag'])) {
                    foreach ($images['tag'] as $key => $tag) {
                        $keywordIds[$keys][] = $imageController->insertKeywordImage($images, $tag);
                    }
                } else {
                    $keywordIds[$keys][] = null;
                }


//                if (isset($images['style'])) {
////                    $title=
////                    $style = Style::create(['title' => ]);
//                    if (isset($imageIds[$keys]))
//                        StyleImage::create(['image_id' => $imageIds[$keys], 'style_id' => $images['style']]);
//                }
                //$insertImageProject = $imageController->insertProjectImage($insertImage->toArray()['id'], $projectId);
            }
//            return 1;
            foreach ($imageIds as $num => $imageId) {
                //dd($keywordIds[3][0]);
                //dd($keywordIds);
                $photo = Image::find($imageId);
                if ($photo) {
                    if ($keywordIds[$num][0] == null) {
                        $photo->keywords()->detach();
                    } else {
                        $photo->keywords()->sync($keywordIds[$num]);
                    }
                }
            }
        }

    }

}
