<?php

namespace App\Http\Controllers;

use App\Http\Requests\EstimateRequest;
use App\Http\Requests\ExternalEstimateRequest;
use App\Http\Requests\NoteRequest;
use App\Http\Resources\MapEstimateResource;
use App\Models\Company;
use App\Models\Estimate;
use App\Models\User;
use App\Notifications\InvoiceNotification;
use App\Notifications\NotificationData;
use App\Repositories\EstimateRepository;
use App\Services\CodeService;
use App\Services\CustomerService;
use App\Services\ListService;
use App\Services\PDFService;
use App\Traits\ApiResponser;
use App\Traits\Helper;
use Illuminate\Http\Request;
use mysql_xdevapi\Exception;
use Symfony\Component\HttpFoundation\Response;

class EstimateController extends Controller
{
    use ApiResponser,NotificationData,Helper;
    public $request;

    public function __construct(Request $request)
    {
        $this->request = $request;
    }

    public function store($clientRole, EstimateRepository $estimateRepository, ListService $listService,
                          CustomerService $customerService, EstimateRequest $estimateRequest, CodeService $codeService)
    {
        $data = $estimateRequest->validated();
//    return 1;
        //add image
        $imageName = null;
        if (array_key_exists('image', $data)) {
            $imageController = new ImageController();
            $imageName = $imageController->upload($estimateRequest->validated()['image']);
        }
        $data['image'] = $imageName;

        $pdfService = new PDFService();
        $pdf=  $pdfService->make($data);
 return $pdf;
//         $dompdf->setPaper('A4', 'landscape');
//        $dompdf->render();
//        $output = $dompdf->output();
//dd($pdf);
        $estimateStore=  $estimateRequest->validated();
        $estimateStore['pdf_address']=$pdf['path'];
//        return $estimateStore;
        $client = auth($clientRole)->user();

        //set referral code
        do {
            $referrenceNumber = $codeService->generateCode();
        } while ($codeService->check(Estimate::class, 'reference_number', $referrenceNumber));


        //add estimate
        $estimate = $estimateRepository->store($estimateStore, $imageName, $referrenceNumber, $client);

        //add list
        foreach ($estimateRequest->validated()['list'] as $list) {
            $listService->addList($list, Estimate::class, $estimate->id, $client);
        }
        $email = new EmailController($this->request);

//       return $email->sendEstimateEmail($data, $pdf);

        try {
            $email = new EmailController($this->request);

            $email->sendEstimateEmail($data, $pdf);

        }catch (\Exception $e){
            return $this->failedResponse('No email was sent');
        }
        //send notification
        $user=null;
        if ($data['customer_email'])
            $user=User::where('email', $data['customer_email'])->first()??Company::where('email', $data['customer_email'])->first();
//        return  $user;
        if ($user)
            $client->notify(New InvoiceNotification($this->notificationData($user,'text','title')));


        return successResponse(true, $estimate->load('lists','estimateable'), 'save estimate successfully');
    }

    public function show($clientRole, $id, EstimateRepository $estimateRepository)
    {
        $client = auth($clientRole)->user();
        $estimate = $estimateRepository->show($id, $client);
//        return $estimate;
        if ($estimate == null){
            return $this->onSuccess([]);
//            return failedResponse('not found any estimate', Response::HTTP_NOT_FOUND);
        }
        return successResponse(true, $estimate, 'ok');
    }
    public function showList($clientRole, EstimateRepository $estimateRepository)
    {
        $client = auth($clientRole)->user();
        $estimate = $estimateRepository->showList($client);
//        return $estimate;
        if ($estimate == null){
            return $this->onSuccess([]);
//            return failedResponse('not found any estimate', Response::HTTP_NOT_FOUND);
        }
        return successResponse(true, $estimate, 'ok');
    }

    public function delete($clientRole, $id, EstimateRepository $estimateRepository, ListService $listService)
    {
        $client = auth($clientRole)->user();
        $estimate = $estimateRepository->show($id, $client);
        if ($estimate == null) {
            return failedResponse('not found any estimate', Response::HTTP_NOT_FOUND);
        }

        $estimateRepository->deleteEstimate($id);
        $listService->delete($id, Estimate::class);

        return successResponse(true, null, 'delete successfully');
    }

    public function addNote($clientRole, $id, NoteRequest $noteRequest, EstimateRepository $estimateRepository)
    {
        $client = auth($clientRole)->user();
        $estimate = $estimateRepository->show($id, $client);
        if ($estimate == null){
            return failedResponse('not found any estimate to add note', Response::HTTP_OK);
        }
        $estimateRepository->addNote($id, $noteRequest->validated());

        return successResponse(true, $estimateRepository->show($id, $client), 'add note successfully');
    }

    public function sendExternalEstimate(ExternalEstimateRequest $externalEstimateRequest,
                                         EmailController $emailController)
    {
        $email=true;
        $data= $externalEstimateRequest;
        $pdfService = new PDFService();
        $pdf=  $pdfService->makeExternal($data);
        $data = $externalEstimateRequest->validated();
//return $externalEstimateRequest->images;
        $imageName = array();
//        if (isset($externalEstimateRequest->images)) {
        if ($externalEstimateRequest['images']){
            $imageController = new ImageController();
            foreach ($externalEstimateRequest->images as $mage){
                $imageName[]=$this->photo($mage);

            }
        }
        $data['image'] = $imageName;
//        return  $imageName;
//        return   $data['image'];
        //        return $externalEstimateRequest;
//        Estimate::query()->create([
//            'customer_name' => $data['name'],
//            'customer_email' => $data['email'],
//            'start_date' => $data['project_start'],
//            'end_date' => $data['project_end'],
//            'reference_number' => $referenceNumber,
//            'estimateable_type' => get_class($client),
//            'estimateable_id' => $client->id,
//            'address' => $data['address'],
//            'message' => $data['message'],
//            'latitude' => $data['latitude'],
//            'longitude' => $data['longitude'],
//            'sub_total' => $data['sub_total'],
//            'discount' => $data['discount'],
//            'discount_type' => $data['discount'],
//            'sale_tax' => $data['sale_tax'],
//            'image' => $imageName,
//            'total' => $data['total']
//        ]);
//        $emailList = [$externalEstimateRequest->email??""];
//        return $externalEstimateRequest;
//        return  $externalEstimateRequest->validated();

//
//                     $emailController->sendExternalEstimate($externalEstimateRequest->email, $externalEstimateRequest->validated());

        try {
            foreach ($externalEstimateRequest->email as $emails){

                   $emailController->sendExternalEstimate2($emails, $data,$pdf);
            }

        }catch (\Exception $e){
            $email=false;
        }

        return (!$email)?$this->failedResponse('No email was sent'):successResponse(true,null,'send successfully');
    }
    public function showMapEstimate($clientRole){

//return 1;
        $client = auth($clientRole)->user();
//return $client->id;
        $map=Estimate::query()
            ->where('estimateable_id',$client->id)
            ->where('estimateable_type', get_class($client))
            ->get();
        return ($map)?$this->onSuccess(MapEstimateResource::collection($map)):$this->onFaile('map not fond');
    }
    public function sendEmailEstimate($clientRole,$id,EstimateRepository $estimateRepository){
        $client = auth($clientRole)->user();
        $estimate = $estimateRepository->show($id, $client);
        if ($estimate == null){
            return failedResponse('not found any estimate to add note', Response::HTTP_OK);
        }
//        return $estimate;
//        return $estimate->id;
//        $email = new EmailController($this->request);
//
//      return  $email->reSendEstimateEmail($estimate, $estimate->pdf_address);
        try {
            $email = new EmailController($this->request);

            $email->reSendEstimateEmail($estimate, $estimate->pdf_address);

        }catch (\Exception $e){
            return $this->failedResponse('No email was sent');
        }
    }



}
